<?php
/**
 * Specific Configuration settings for WP-Favicons
 * @package WP-Favicons
 * @since 0.4.0
 * @author Edward de Leau <e@leau.net>, http://edward.de.leau.net
 * @copyright GPL 2
 */
namespace leau\co\wp_favicons;

/* my generic plugin framework */
require_once dirname( __FILE__  ) . '/class-config.php';
require_once dirname( __FILE__  ) . '/class-utils.php';
require_once dirname( __FILE__  ) . '/tld/class-database.php';
require_once dirname( __FILE__  ) . '/class-module.php';
require_once dirname( __FILE__  ) . '/class-modules.php';
require_once dirname( __FILE__  ) . '/class-plugin.php';

/* Specific includes needed for this plugin */
$plugin_root = dirname( __FILE__ ) . '/../plugins/';
require_once $plugin_root . '/cache/inc/class-EdlCache.php';
require_once $plugin_root . '/context/inc/class-EdlHtmlRef.php';
require_once $plugin_root . '/context/inc/class-EdlLinkFaviconContent.php';
require_once $plugin_root . '/context/inc/class-module-context-plugin.php';
require_once $plugin_root . '/filters/inc/class.ico.php';

if (!class_exists("LoadConfiguration"))
{
	/**
	 * Sets Specific Configuration settings for WP-Favicons
	 * uses the generic config class
 	 * @since 0.4.0
     * @author Edward de Leau <e@leau.net>, http://edward.de.leau.net
	 *
	 */
	class LoadConfiguration
	{
		function __construct()
		{
			// handy vars
			$page_title = 'page_title';
			$menu_title = 'menu_title';
			$menu_slug = 'menu_slug';
			$section_header_title = 'section_header_title';
			$section_header_text = 'section_header_text';
			$help_text = 'help_text';
			$gurl='http://www.google.com/s2/favicons?domain=';

			// set the specific configuration for the favicon plugin:
			Config::SetOptionsName('wp_favicons_options');
			Config::SetOptionsFromArray(get_option(Config::GetOptionsName()));
			Config::SetSettingsGroupName('wp_favicons_settings_group');
			Config::SetPageTitle('WP Favicons Settings');
			Config::SetMenuTitle('WP-Favicons');
			Config::SetCapability('manage_options');
			Config::SetMenuSlug('WP-Favicons-Menu');
			Config::SetPluginSlug('wp_favicons_');
			Config::AddMenuIcon(plugins_url('/img/wpfavicons.png', __FILE__));
			Config::SetMainPage(
			__('<h2>Welcome to the WordPress Favicons plugin!</h2>' .

			'<p>This <img src="'. $gurl . 'wordpress.org"> WordPress Plugin adds icons'.
			' in front of your weblinks</p>' .

			'<p>There are many reasons why this is better than no icons, but I will leave it up ' .
			'to you to write a blogpiece about that "why".<br />If you send the link, I will link it here.</p>' .

			'<h3>About the previous Version (0.3) </h3>'.
			'The previous release was still immature and the biggest fault was that '.
			'directories were written with rights 0700 instead of 0755 resulting in '.
			'the wrong right in the cache '.
			'<h3>About this release (0.4.0 Beta)</h3>'.
			'I have changed a lot for this release. Probably too much to mention but '.
			'lets say the underlying code is now ready for making much more enhancement '.
			'much quicker. Some cool things: '.
			'<ul>' .
			'<li>- It has a plugin framework so that most of the part is now configured ' .
			'by one configuration set. If you are a developer you may resuse it, enhance '.
			'it and provide me improvement since it can be used for any plugin</li>'.
			'<li>- ALL functionality is now in "modules" and "plugins" (probably plugins is a '.
			'bad word) and these get loaded on the fly. The modules and plugins are open so '.
			'that a developer can add modules and plugins to enhance the functionality</li>'.
			'<li>- The favicons are now sought from various source since Google is very limited '.
			'in the icons it offers. Though the first loads of pages get be slow to get the '.
			'cache filled... it works (if there are LOTS of icons it might take a reload but '.
			'ofcourse only on the first load. A next version will try preloading and/or seek ' .
			'for performance improvements on this. A next version will also allow you set up '.
			'your own favicon service with just a shortcode.</li>' .
			'<li>- There is no longer a need to replace icons in the cache and you should not '.
			'(since the cache might get deleted). You can now write filters to either specificy '.
			'default icons and/or filter the graphics e.g. darken them on the fly.<li>'.
			'<li>- There is now a database running in the background which is the beginning '.
			'of an external uri metadata framework and which stores URIs in a hierachical way. '.
			'There are a lot of cool things to add to this in next versions, you will truly '.
			'master your external links or even start your own (yoursite) Social Graph. '.
			'However init of the database takes some time while installing the plugin '.
			'I will make that different in a next release</li>'.
			'<li>- Internationalization has been added although I did not have time to work on '.
			'all the bits to translate it completely yet, since there are still code changes.'.
			'The good thing is that most core strings are now in one configuration file.</li>'.
			'<li>- Default WordPress plugin things like activation hooks, deactivation, '.
			'and such have been added as well as PHPDoc comments in all code to make all of it '.
			'very readable. If you run PHPDoc on the code you will not get perfect output yet '.
			'this is also to improve upon in next releases.</li>'.
			'<h3>Modules and plugins</h3>' .
			'In this release I made several improvement in the backend. ' .
			'The architecture now consists of several modules, and each module ' .
			'is build out of plugins. </p>' .
			'<img src="' . WP_PLUGIN_URL . '/wp-favicons/includes/img/img1.png">' .
			'<p>Each of the blocks above corresponds to a module which corresponds also ' .
			'to a page you can configure in the administration menu ' .
			'(and if you are a developer you can even add extra modules to the workflow).' .
			'</p>' .
			'<ul>' .
			'<li>' .
			'<b>- Content</b>: WordPress consists out of many elements e.g. Posts, different kind ' .
			'of widgets and other elements. For each of the elements you can define a context item ' .
			'and style it differently. I have provided some default filters but if you are a developer ' .
			'you might e.g. just capture all php output for a sidebar and turn that in a plugin to add ' .
			'to the context menu or think of your custom widgets.' .
			'</li>'.
			'<li>' .
			'<b>- Links</b>: actually... we should call these URI\'s... Each of the content items ' .
			'contains multiple links. In this release we focus on favicons for these URI\'s in following ' .
			'releases we will add support for more concepts. ' .
			'</li>'.
			'<li>' .
			'<b>- Exceptions</b>: you might want to exclude some links, this is handled in the exceptions ' .
			'module, default I delivered 1 plugin: to exclude certain file extensions. ' .
			'</li>'.
			'<li>' .
			'<b>- Sources</b>: Favicons may come from many sources such as the Google Service, other ' .
			'third party providers such as geticon.org, a favicon.ico in the root or elsewhere mentioned ' .
			'on pages. By default I include 4 icon providers. Developers can add more plugins for this '.
			'</li>'.
			'<li>' .
			'<b>- Default Image</b>: If none of the providers can deliver a favicon well... maybe the site '.
			'has no icon or it is not available from the outside (such as Facebook application icons) ' .
			'you might also think of icons that are related to shortlinked urls though for any of these ' .
			'extra providers may be included. Anyway, in that case a default icon is needed. By default ' .
			'I include 1 plugin for this, developers can add more default options ' .
			'</li>'.
			'<li>' .
			'<b>- Filters</b>: On the resulting icon many image filtering may take place. Sharpen it, ' .
			'round corner it, darken it, whatever. Default I convert all images to PNG. Please do not disable '.
			'that plugin for now, since the display code expects a png, that is @todo' .
			'</li>'.
			'<li>' .
			'<b>- Cache</b>: Icons are stored in your upload directory and eternally cached. They follow ' .
			'a handy reversed domain directory structure e.g. /com/google/www/favicon.ico where you can find ' .
			'them if you want them for other operations. Please do not replace icons in here since there ' .
			'is also the pluggin "delete cache" .... One thing to realize is that the icons are stored on ' .
			'domain level, so that every icon is stored on domain level and not on relative directory level ' .
			'with one exception: Facebook (for Facebook apps). I will try to enhance that a bit in coming  ' .
			'releases but that is how it works now and well... that strategy works for most sites. The ' .
			'alternative is so complicated that I can not summarize it here</li>'.
			'<li><b>- Presentation</b>: this module will be added in a next release.</li>'.
			'</ul>' .
			'<p>After following this order the icon is displayed in the context on the WordPress Screen ' .
			'ofcourse, when it is in the cache, it will grab it directly from the cache. But notice that ' .
			'when an icon has not been cached yet... it takes some time longer to load the page.</p>' .

			'<h3>URIs and metadata</h3>' .
			'Although it sounds simple. Getting and storing a favicon can be a complicated thing '.
			'e.g. www.abc.com and abc.com can have different icons as well as abc:81.com and abc:80.com '.
			'as well as abc.com/foo/bar/page1.php versus abc.com/foo/bar/page2.php. The icons service '.
			'of Google has taken a shortcut and simply looks at favicon.ico in the root. But since this '.
			'is THE favicon plugin, this one should do better. Since I only want to store favicons and '.
			'later on, other meta information only once, I need to determine in the family tree of a set '.
			'of URIs what the most common ancestor is to hold a unique set from the viewpoint of certain '.
			'piece of metadata. E.g. the Microsoft domain is split by domains, Youtube uses a path for ' .
			'unique user domains, Wordpress MU sites can be either domain based on directory based. The favicon '.
			'will be unique either per subdomain or per pathname. For that I include a database with the '.
			'purpose of capturing this family tree of URI information and to attach metadata on in ' .
			'later stages and to allow developers to add much more information to it. '.
			'In this release a foundation of the TLD structure is added (although unfortunately that '.
			'takes some time to init) (I might move that activation to here). With the future in mind: '.
			'Downloading the Alexa top 1.000.000 sites and attach the Alexa ranking to each node gives '.
			'a good test set. Unfortunately that is too big to supply with the plugin, maybe I will '.
			'make a loader for that in the next release'.
			'<h3>Dummy Mode</h3>' .
			'If the cache is turned off you are in dummy mode meaning: icons will simply be grabbed ' .
			'from Google :)' .
			'<h3>TODO</h3>' .
			'I have commented @todo in the code so if you dump that list you will see the things '.
			'that should be ready for the next release. Please report bugs and wishes in the forum so I '.
			'work on them also. For this release I worked more on the infrastructure on it, the next' .
			'release will improve on the replacement set, add the presentation module and probably '.
			'will add some more plugins.' )
			);
			/* ---------------------- module: context ----------------------*/
			$moduleID = 'context';
			/* main module settings */
			Config::AddModule(array(
				$page_title 			=> 'Favicon Replacement Context',
				$menu_title 			=> 'Context',
				$menu_slug 				=> 'context',
				$section_header_title 	=> '<h2>Favicons Context</h2>',
				$section_header_text   =>
					__('A wordPress theme consists out of many elements. Each element has its own demands. On '.
					'this page you select where you want to have icons added.'),
				$help_text				=>
					__('<h3>The WordPress Favicons Plugin - Context Screen</h3>' .
					'<p>A WordPress theme consists of many components. Each component ' .
					'can be treated differently and styled differently.</p> <p> On a functional ' .
					'level you might want big icons in your post content and small icons ' .
					'in a sidebar widget. Or... you might even want different styled icons (or no) icons per ' .
					'widget. This can be set on this page.</p>' .
					'<p>On a technical level widgets and other content provide different \'filters\': ' .
					'each of the filters can be called and used differently.</p>' .
					'<p>If you are a developer you can add plugins to this page to add more ' .
					'content replacers. You might e.g. write a plugin for replacing the content ' .
					'of a complete sidebar using ob_. The architecture is open so it is easy to ' .
					'add this plugin so they show up here.</p>' .
					'<h3>Specific information per Wp-Favicons Content plugin:</h3>')
			));
			/* plugin: the_content */
			$pluginID = 'the_content';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				__('Adds a favicon in the post and page content','wp-favicons'));
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				__('Posts and Pages','wp-favicons'));
			Config::SetModulePlugin($moduleID,$pluginID,'filter_context_name',
				'the_content');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'show_in_posts',
    				'default' => 1,
    				'label' => __('Replace in posts','wp-favicons'),
    				'type' => 'radio_bool'));
			Config::SetModulePluginField($moduleID,$pluginID,1,
				array(	'name' => Config::GetPluginSlug() . 'show_in_posts_css',
    				'default' => 'width:10px!important;height:10px!important;margin-left:0px!important;margin-right:0px!important;padding-left:3px!important;padding-right:3px!important;padding-bottom: 0px!important;padding-top:0px!important;',
	    			'label' => __('Styling','wp-favicons'),
    				'type' => 'textarea'));
			/* plugin: widget_text */
			$pluginID = 'widget_text';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				'Adds a favicon in text areas of widgets');
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Text areas of widgets');
			Config::SetModulePlugin($moduleID,$pluginID,'filter_context_name',
				'widget_text');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'show_widget_text',
    				'default' => 1,
    				'label' => 'Replace Widget Text',
    				'type' => 'radio_bool'));
			Config::SetModulePluginField($moduleID,$pluginID,1,
				array(	'name' => Config::GetPluginSlug() . 'show_in_widget_text_css',
    				'default' => 'width:10px!important;height:10px!important;margin-left:0px!important;margin-right:0px!important;padding-left:3px!important;padding-right:3px!important;padding-bottom: 0px!important;padding-top:0px!important;',
	    			'label' => 'Styling',
    				'type' => 'textarea'));
			/* plugin: bookmark_list */
			$pluginID = 'bookmark_list';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				'Adds a favicon in bookmark list widgets');
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Bookmark List widgets');
			Config::SetModulePlugin($moduleID,$pluginID,'filter_context_name',
				'wp_list_bookmarks');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'show_in_list_bookmarks',
    					'default' => 1,
    					'label' => 'Replace in Bookmark Lists',
    					'type' => 'radio_bool'));
			Config::SetModulePluginField($moduleID,$pluginID,1,
				array(	'name' => Config::GetPluginSlug() . 'show_in_list_bookmarks_css',
    					'default' => 'width:10px!important;height:10px!important;margin-left:0px!important;margin-right:0px!important;padding-left:3px!important;padding-right:3px!important;padding-bottom: 0px!important;padding-top:0px!important;',
	    				'label' => 'Styling',
    					'type' => 'textarea'));
			/* plugin: comment text */
			$pluginID = 'comment_text';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				'Adds a favicon in comment text');
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Comment Text');
			Config::SetModulePlugin($moduleID,$pluginID,'filter_context_name',
				'comment_text');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'show_in_comment_text',
    					'default' => 1,
    					'label' => 'Replace in Comment Text',
    					'type' => 'radio_bool'));
			Config::SetModulePluginField($moduleID,$pluginID,1,
				array(	'name' => Config::GetPluginSlug() . 'show_in_comment_text_css',
    					'default' => 'width:10px!important;height:10px!important;margin-left:0px!important;margin-right:0px!important;padding-left:3px!important;padding-right:3px!important;padding-bottom: 0px!important;padding-top:0px!important;',
	    				'label' => 'Styling',
    					'type' => 'textarea'));

			/* ---------------------- module: sources ----------------------*/
			$moduleID = 'sources';
			/* main module settings */
			Config::AddModule(array(
				'page_title' => 'Favicon sources',
				'menu_title' => 'Sources',
				'menu_slug' => $moduleID,
				'section_header_title' => '<h2>Favicons Sources</h2>',
				'section_header_text' =>
					"There are multiple sources to retrieve the favicon from. ".
					"Select the ones you would like to use. Please note that with each addition, ".
					"depending on the provider, your page load will decrease. However, IF you have the ".
					"WP-Favicons Cache on this will happen only the first time it needs to look for an ".
					"icon. The benefit of more providers is that you will increase your chance of finding ".
					"the icon.",
				'help_text' =>
					'<h3>The WordPress Favicons Plugin - Context Screen</h3>' .
					'<p>A WordPress theme consists of many components. Each component ' .
					'can be treated differently and styled differently.</p> <p> On a functional ' .
					'level you might want big icons in your post content and small icons ' .
					'in a sidebar widget. Or... you might even want different styled icons (or no) icons per ' .
					'widget. This can be set on this page.</p>' .
					'<p>On a technical level widgets and other content provide different \'filters\': ' .
					'each of the filters can be called and used differently.</p>' .
					'<p>If you are a developer you can add plugins to this page to add more ' .
					'content replacers. You might e.g. write a plugin for replacing the content ' .
					'of a complete sidebar using ob_. The architecture is open so it is easy to ' .
					'add this plugin so they show up here.</p>' .
					'<h3>Specific information per Wp-Favicons Content plugin:</h3>'
					));

			/* plugin: extract from page */
			/* 0.4.5: fix for the order: many icons are not picked up by google
			 * therefore an in page scan assures that we DO get the correct icon
			 * e.g. google returns all facebook app pages as facebook icon
			 * */
			$pluginID = 'page';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				'Makes a call to the host links in the html code to a favicon (icon,shortcut icon)');
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Check page source');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'search_page',
    					'default' => 0,
    					'label' => 'Search in Page',
    					'type' => 'radio_bool'));
			/* plugin: extract from root */
			$pluginID = 'root';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				'Makes a call to the host to check for /favicon.ico in the root');
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Check /favicon.ico');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'search_root',
    					'default' => 0,
    					'label' => 'Search Root',
    					'type' => 'radio_bool'));
			/* plugin: Google S2 */
			$pluginID = 'google_s2';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				'Uses Google to find favicons in .png format, this is very fast');
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Search Google');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'search_google',
    					'default' => 1,
    					'label' => 'Search Google',
    					'type' => 'radio_bool'));
			/* plugin: geticonorg */
			$pluginID = 'geticonorg';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				'Uses geticon.org to find favicons in .png format, nice backup for Google');
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Search Favicon.org');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'search_geticonorg',
    					'default' => 0,
    					'label' => 'Search Geticon.org',
    					'type' => 'radio_bool'));


			/* ---------------------- module: cache ----------------------*/
			$moduleID = 'cache';
			/* main module settings */
			Config::AddModule(array(
				'page_title' => 'Favicon cache',
				'menu_title' => 'Cache',
				'menu_slug' => $moduleID,
				'section_header_title' => '<h2>Caching functionality</h2>',
				'section_header_text' =>
					"Much of the functionality of this plugin is turned on when you activate the cache. ".
					"The cache is created on your server and downloads and creates and convert images in there. If you ".
					"select a lot of options, first loads may be slow but after it is in the cache all further ".
					"page requests should go pretty fast. If you do not use the cache it will simply use Google ".
					"for the image."	,
				'help_text' => "Caching functionality"
			));
			/* plugin: cache settings */
			$pluginID = 'settings';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				'To make this simple: Use the Cache or just use Google');
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Use the cache');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'use_cache',
    					'default' => 0,
    					'label' => 'Use the cache',
    					'type' => 'radio_bool'));
			/* plugin: empty cache */
			$pluginID = 'empty_cache';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			if (Config::GetOptionsArrayValue('cache_emptied_date'))
			{
				Config::SetModulePlugin($moduleID,$pluginID,'header',
					'Empty the current cache. After emptying the value will jump back to no and the time will be recorded. This option cleans the physical disk cache as well as the database entries. Last time Emptied on: ' .
						Config::GetOptionsArrayValue('cache_emptied_date'));
			}
			else
			{
				Config::SetModulePlugin($moduleID,$pluginID,'header',
					'Empty the current cache. After emptying the value will jump back to no and the time will be recorded. This option cleans the physical disk cache as well as the database entries. ');
			}

			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Empty the cache');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'empty_cache',
    					'default' => 0,
    					'label' => 'Empty the cache',
    					'type' => 'radio_bool'));

			/* ---------------------- module: filters ----------------------*/
			$moduleID = 'filters';
			/* main module settings */
			Config::AddModule(array(
				'page_title' => 'Favicon filters',
				'menu_title' => 'Filters',
				'menu_slug' => $moduleID,
				'section_header_title' => '<h2>Filters</h2>',
				'section_header_text' =>
					"<ul>".
					"<li>Filters allow image manipulation after retrieval of the icon.<li>" .
					"<li>You need to have the cache on to enable these filters.</i>".
					"<li>If you changed filters and want to apply them to existing cached icons then empty the cache.</i>".
					"</ul>",
				'help_text' => "Filters Help"
			));
			/* plugin: convert to png */
			$pluginID = 'convert_to_png';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				'Converts all received icons to <a href="http://en.wikipedia.org/wiki/Portable_Network_Graphics" target="_blank">png</a>');
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Convert to PNG');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'convert_to_pngs',
    					'default' => 1,
    					'label' => 'Convert to PNG',
    					'type' => 'radio_bool'));

			/* ---------------------- module: default ----------------------*/
			$moduleID = 'default';
			/* main module settings */
			Config::AddModule(array(
				'page_title' => 'Default icons',
				'menu_title' => 'Default icon',
				'menu_slug' => $moduleID,
				'section_header_title' => '<h2>Default icon</h2>',
				'section_header_text' =>
			 		"<ul>".
					"<li>If no image is found by the plugins you activated under 'sources' a default icon will be used.</li>".
					"<li>If you select multiple defaults only one will be used (the top one).</li>".
					"<li>If all plugins hereunder are disabled then nothing will be shown if no icon is found.</li>".
					"<li>If you change to another default and want already cached default icons to be resetted then empty the cache.<li>",
				'help_text' => "Default Icon Help"
			));
			/* plugin: identicon */
			$pluginID = 'identicon';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				'An Identicon is a visual representation of a hash value of the IP address of the website.');
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Shows an <a href="http://en.wikipedia.org/wiki/Identicon" target="_blank">identicon</a> when no favicon is found');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'default_identicon',
    					'default' => 1,
    					'label' => 'Show identicon',
    					'type' => 'radio_bool'));


			/* ---------------------- module: filetypes ----------------------*/
			$moduleID = 'filetypes';
			/* main module settings */
			Config::AddModule(array(
				'page_title' => 'File Types',
				'menu_title' => 'File Types',
				'menu_slug' => $moduleID,
				'section_header_title' => '<h2>FileTypes</h2>',
				'section_header_text' =>
					"Later versions will be able to assign icons to filetypes For now you can just add filetypes to ignore for favicon display",
				'help_text' =>
					"Later versions will be able to assign icons to filetypes For now you can just add filetypes to ignore for favicon display"
			));
			/* plugin: exclude filetypes */
			$pluginID = 'excluded_filetypes';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				'Filetypes that are excluded');
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Excluded Filetypes');
			Config::SetModulePluginField($moduleID,$pluginID,0,
				array(	'name' => Config::GetPluginSlug() . 'exclude_filetypes',
    					'default' => '.zip .pdf .arj',
    					'label' => 'Exclude the following',
    					'type' => 'text'));

			/* ---------------------- module: information ----------------------*/
			$moduleID = 'information';
			/* main module settings */
			Config::AddModule(array(
				'page_title' => 'Information',
				'menu_title' => 'Information',
				'menu_slug' => $moduleID,
				'section_header_title' => '<h2>Information</h2>',
				'section_header_text' =>
					"Show you technical information",
				'help_text' =>
					"Show you technical information"
			));
			/* plugin: directories */
			$pluginID = 'directories';
			Config::RegisterModulePlugin($moduleID,$pluginID);
			Config::SetModulePlugin($moduleID,$pluginID,'header',
				'(read only)');
			Config::SetModulePlugin($moduleID,$pluginID,'title',
				'Directories and files');
			$upload_dir = wp_upload_dir();
			$rootUpload_dir = 'wp-favicons';
			$rootUpload_dirFull =  $upload_dir['basedir'] . '/' . $rootUpload_dir . '/';
			$rootUpload_urlFull =  $upload_dir['baseurl'] . '/' . $rootUpload_dir . '/';
			$cssUpload_dir = 'style/';
			$cssUpload_file = 'style.css';
			$cacheUpload_dir = 'cache/';
			$n=0;
			Config::SetModulePluginField($moduleID,$pluginID,$n++,
				array(  'name' 		=> Config::GetPluginSlug() . 'upload_dir',
						'default' 	=> $rootUpload_dirFull,
						'label' 	=> 'Physical Upload Directory',
    					'type' 		=> 'text_no_change'));
			Config::SetModulePluginField($moduleID,$pluginID,$n++,
				array(	'name' 		=> Config::GetPluginSlug() . 'upload_dir_style',
						'default' 	=> $rootUpload_dirFull . $cssUpload_dir,
						'label' 	=> 'Physical Stylesheet Directory',
    					'type' 		=> 'text_no_change'));
			Config::SetModulePluginField($moduleID,$pluginID,$n++,
				array(	'name' 		=> Config::GetPluginSlug() . 'upload_dir_cache',
						'default' 	=> $rootUpload_dirFull . $cacheUpload_dir,
						'label' 	=> 'Physical Cache Directory',
    					'type' 		=> 'text_no_change'));
			Config::SetModulePluginField($moduleID,$pluginID,$n++,
				array(	'name' 		=> Config::GetPluginSlug() . 'upload_dir_style_file',
						'default' 	=> $rootUpload_dirFull . $cssUpload_dir . $cssUpload_file,
						'label' 	=> 'Physical Stylesheet Location',
    					'type' 		=> 'text_no_change'));
			Config::SetModulePluginField($moduleID,$pluginID,$n++,
				array(	'name' 		=> Config::GetPluginSlug() . 'upload_url',
						'default' 	=> $rootUpload_urlFull,
						'label' 	=> 'Relative root',
	    				'type' 		=> 'text_no_change'));
			Config::SetModulePluginField($moduleID,$pluginID,$n++,
				array(	'name' 		=> Config::GetPluginSlug() . 'upload_url_style',
						'default' 	=> $rootUpload_urlFull . $cssUpload_dir,
						'label' 	=> 'Relative CSS Location',
    					'type' 		=> 'text_no_change'));
			Config::SetModulePluginField($moduleID,$pluginID,$n++,
				array(	'name' 		=> Config::GetPluginSlug() . 'upload_url_style_file',
						'default' 	=> $rootUpload_urlFull . $cssUpload_dir . $cssUpload_file,
						'label' 	=> 'Relative CSS file location',
    					'type' 		=> 'text_no_change'));
			Config::SetModulePluginField($moduleID,$pluginID,$n++,
				array(	'name' 		=> Config::GetPluginSlug() . 'upload_url_cache',
						'default' 	=> $rootUpload_urlFull . $cacheUpload_dir,
						'label' 	=> 'Relative Cache location',
    					'type' 		=> 'text_no_change'));

		}
	}
}
new LoadConfiguration;