<?php
/**
 * Contains Util Classes.
 *
 * General Utilities
 * @package WP-Favicons
 * @since 0.4.0
 * @author Edward de Leau <e@leau.net>, http://wp.leau.co
 * @copyright GPL 2
 */

namespace leau\co\wp_favicons;


if (!class_exists("EdlUtils"))
{

	/**
	 * Class: EdlUtils
	 *
	 * @since 0.1
	 * @author Edward de Leau <e@leau.net>
	 * @copyright GPL 2
	 * @static class containing utility functions
	 */
	class EdlUtils
	{
		CONST LOGLEVEL = 3;

		// singleton instance
		private static $instance;

		// private constructor function
		// to prevent external instantiation
		private function __construct()
		{

		}
		private function __clone()
		{
		}

		// getInstance method
		public static function getInstance()
		{
			if(!self::$instance)
			{
				self::$instance = new self();
			}
			return self::$instance;
		}

		/**
		 * Writes a file
		 * Writes a file to a directory
		 * @param $dir
		 * @param $file
		 * @param $content
		 */
		public static function WriteFile($dir,$file,$content)
		{
			if (!is_dir($dir)) {
				mkdir($dir, 0755, true);
			}
			$fh = fopen($file, 'w') or die("can't open file");
			fwrite($fh, $content);
			echo "<!-- " . $file ." written -->";
			fclose($fh);
		}

		// from http://php.net/manual/en/function.serialize.php
		/**
		 * converts an array to string
		 * @param $array
		 * @param $level
		 */
		function array2str($array,$level=1)
		{
			$str = array();
			foreach($array as $key=>$value)
			{
				$nkey = base64_encode($key);
				$nvalue = is_array($value)?'$'.base64_encode(array2str($value)) : (string)base64_encode($value);
				$str[] = $nkey.'&'.$nvalue;
				//	printf("Encoded %s,%s to %s,%s\n",$key,$value,$nkey,$nvalue);
			}
			return implode('|',$str);
		}

		/**
		 * converts a string to an array
		 * @param $str
		 */
		function str2array($str)
		{
			$rest = array();
			if(strpos($str,'|')>0)
			{
				$array = explode('|',$str);
			}
			else
			{
				$array=array($str);
			}
			foreach($array as $token)
			{
				list($key,$value) = explode('&',$token);
				$nkey=base64_decode($key);
				$nvalue = (substr($value,0,1) != '$' ? base64_decode($value) : str2array(base64_decode(substr($value,1))) );
				$rest[$nkey] = $nvalue;
				//printf("Decoded %s,%s to %s,%s\n",$key,$value,$nkey,$nvalue);
			}
			return $rest;
		}

		/**
		 * Some admin fields we use currently
		 * Will add some more in later releases
		 * @param $field
		 * @param $fieldname
		 * @param $fieldvalue
		 * @uses checked as defined in \wp-includes\general-template.php
		 *
		 * @todo 0.5.0 refine e.g. esc and extend these helpers with more fields
		 */
		function AdminForm($field,$fieldname,$fieldvalue)
		{
			switch ($field['type']) {
				case 'radio_bool':
					echo '<input ';
					echo 'type="radio" ';
					echo 'name="' . $fieldname . '" ';
					echo 'id="' . $fieldname . '" ';
					echo ' value="1" ';
					checked( '1', $fieldvalue );
					echo ' /> yes <br />';

					echo '<input ';
					echo 'type="radio" ';
					echo 'name="' . $fieldname . '" ';
					echo 'id="' . $fieldname . '" ';
					echo ' value="0" ';
					checked( '0', $fieldvalue );
					echo " /> no";
					break;
				case 'text':
					echo '<input ';
					echo 'type="text" ';
					echo 'name="' . $fieldname . '" ';
					echo 'id="' . $fieldname . '" ';
					echo 'value="' . $fieldvalue . '" ';
					echo 'size="150" ';
					echo " />";
					break;
				case 'text_no_change':
					echo '<input ';
					echo 'type="text" ';
					echo 'name="' . $fieldname . '" ';
					echo 'id="' . $fieldname . '" ';
					echo 'value="' . $fieldvalue . '" ';
					echo 'size="150" disabled';
					echo " />";
					break;
				case 'textarea':
					echo '<textarea ';
					echo 'name="' . $fieldname . '" ';
					echo 'rows="5" ';
					echo 'cols="80" ';
					echo '>';
					echo $fieldvalue;
					echo '</textarea>';
					break;
			}

		}

		/**
		 * Outputs a message in the screen output
		 * Handy for debugging
		 * @param unknown_type $url
		 * @param unknown_type $msg
		 */
		function favicon_page_msg($url,$msg) {
			if (is_user_logged_in() && current_user_can('activate_plugins'))
			{
				echo "\n<!-- [WP-FAVICONS]:";
				echo $url;
				echo " " . $msg;
				echo " -->\n";
			}
		}

		/**
		 *
		 * Enter description here ...
		 * @param $favicon_array
		 * @param $url
		 */
		function favicon_filter_init( /*array*/ $favicon_array, /*string*/ $url)
		{
			/* always check that no icon has been found yet */
			$favicon_found = false;
			if ($favicon_array)
			{
				if (array_key_exists('data', $favicon_array) &&
						'' != $favicon_array['data'])
				{
					return false;
				}
			}

			// testing larger time-out
			if( !ini_get('safe_mode') ){
				//echo ini_get('max_execution_time');
				ini_set('max_execution_time',1000);
				set_time_limit(1000);
			}

			//add_filter( 'http_request_timeout', 5)
			//add_filter( 'http_request_redirection_count', 5)
			//add_filter( 'http_request_version', '1.0')
			//add_filter( 'http_headers_useragent', 'WordPress/' . $wp_version . '; ' . get_bloginfo( 'url' )  )

			/* otherwise get it */
			$response = wp_remote_get($url);
			if ( is_wp_error( $response ))
			{
				$error_string = $response->get_error_message();
				//echo "<!-- [WP-FAVICONS]: " .  $error_string  . " -->";
				EdlUtils::favicon_page_msg('','[DL] ' .
						$error_string);
				return false;
			}
			else
			{
				//
				// @todo 0.5.0 check for images and pages return
				// error codes in ico format for error pages
				// with option to strike the corresponding URI
				//
				// @todo 0.5.0 sometimes we get redirected to a html page
				// which HAS a favicon but then even in the page itself...
				// e.g. type newscred.com -> forwards to platform.newscred.com
				// --> /favicon.ico ---> forwards to newspaper.newscred.com
				// uhm...
				if ($response['response']['code'] != 200)
				{
					EdlUtils::favicon_page_msg('','[DL] ' .
						$response['response']['code']);
					//echo "<!-- ";
					//print_r($response['headers']);
					//print_r($response['response']);
					//echo " -->";
					return false;
				}
				else
				{
					EdlUtils::favicon_page_msg('','[DL] ' .
						$response['response']['code']);
					//echo "<!-- ";
					//print_r($response['headers']);
					//print_r($response['response']);
					//echo " -->";
					$data = $response['body'];
				}
			}

			if (!$data)
			{
				return false;
			}
			return $data;
		}

		/**
		 * Gets a file extension
		 * Centralized since e.g. wordpress.org uses favicon.ico?3 meaning the
		 * ?3 got in the extension... better to centralize this.
		 * @since 4.4.6
		 */
		function GetExtension($imagePath)
		{
			$ext = end(explode(".",$imagePath));
			/* @todo 0.5.0 extension: down know if this works in all cases
			* we do not even need an extension since the png converter recognizes
			* the image format and writes it however... the user CAN have it
			* turned off .... so....
			* It could also be that a website does include a "favicon.php" which
			* holds data (even if it is bogus)
			* The resolution is probably to split the PNG plugin in 2 parts:
			* one part checks the content of the data and if it is an image
			* the other part does a png conversion
			* we then makes the first part required to prevent bogus data written
			* so we disable the option to disable it in the plugin settings
			* however that is for version 0.5 this is a quick fix:
			*/
			if ($ext != "jpeg" || $ext='wbmp')
			{
				$ext = substr($ext,0,3);
			}
			return $ext;

		}


		/**
		 * Gets the filetype of an image
		 * @param $data
		 * @param $filetype
		 */
		function GetFileType($data, $filetype)
		{
			/* we need a physical file to determine file type */
			$admin_options = get_option('wp_favicons_options');
			$tempfilename = 'tempfavicon';
			$tempdir = Config::GetOptionsArrayValue('wp_favicons_upload_dir_cache');
			$tempfile = $tempdir . '/' . $tempfilename;
			EdlUtils::WriteFile($tempdir, $tempfile,  $data);
			EdlUtils::favicon_page_msg('','[VERIFY] '. $tempdir . $tempfilename . ' written');

			/* check type via exif_imagetype */
			$filetype='';
			if ( ! function_exists( 'exif_imagetype' ) )
			{
				if ( ( list($width, $height, $type, $attr) = getimagesize( $tempfile ) ) !== false )
				{
					$type_to_check = $type;
				}
				else
				{
					EdlUtils::favicon_page_msg('','[VERIFY] no PHP EXIF and no PHP IMAGETYPE');
					$type_to_check = '';
				}
			}
			else
			{
				$type_to_check = exif_imagetype($tempfile);
			}

			switch($type_to_check)
			{
				case IMAGETYPE_ICO:
					$filetype = 'ico';
					break;
				case IMAGETYPE_GIF:
					$filetype = 'gif';
					break;
				case IMAGETYPE_JPEG:
					$filetype = 'jpg';
					break;
				case IMAGETYPE_PNG:
					$filetype = 'png';
					break;
				case IMAGETYPE_ICO:
					$filetype = 'png';
					break;
				case IMAGETYPE_BMP:
					$filetype = 'bmp';
					break;
				case IMAGETYPE_XBM:
					$filetype = 'xbm';
					break;
				case IMAGETYPE_WBMP:
					$filetype = 'wbmp';
					break;
				default:
					EdlUtils::favicon_page_msg('','[VERIFY] '. $type_to_check . ' found however: not supported');
					$filetype = '';
					break;
			}
			return $filetype;
		}

		/**
		 * (see comments on GetExtension)
		 * We centralize the image check and type functionality so that a plugin
		 * can call it to verify the contents (this is part one)
		 */
		function VerifyImage( /*array*/ $favicon_array )
		{
			if ( ! function_exists('imagecreatefromstring') )
			echo '<!-- The GD image library is not installed. -->';

			if (!$favicon_array)
			{
				$favicon_array='';
			}

			/* if it contains html stuff  ditch it*/
			if (strpos(strtolower("-".$favicon_array['data']),strtolower('<html')) &&
				strpos(strtolower("-".$favicon_array['data']),strtolower('<body'))) {
					$favicon_array='';
			}

			/* if plugin indicates that it is really a png */
			if (array_key_exists('type',$favicon_array) && 'pngsure' == $favicon_array['type'])
			{
				EdlUtils::favicon_page_msg('',
						'[VERIFY] not checked - since this was indicated as png');
						$favicon_array['type'] = 'png';
			}
			/* if ICO for sure: convert and return */
			elseif (array_key_exists('type',$favicon_array) && 'icosure' == $favicon_array['type'])
			{
				EdlUtils::favicon_page_msg('',
						'[VERIFY] not checked - since this was indicated as ico');
						$favicon_array['type'] = 'ico';
			}
			else
			{
				/* determine the filetype */
				EdlUtils::favicon_page_msg('','[VERIFY] check image type');
				$filetype = EdlUtils::GetFileType($favicon_array['data'], $favicon_array['type']);
				$favicon_array['type'] = $filetype;
				/* check that it is an image */
				/*
				$image='';
				$image = @imagecreatefromstring($favicon_array['data']);
				if ( !is_resource( $image ) )
				{
					$favicon_array='';
				}
				else
				{
					// @0.5.0 not (yet) supported filetype
					$favicon_array='';
				}*/
			}
			return $favicon_array;
		}
	}
}