<?php

/**
 * Database functions for wp-favicons
 * This php file has not been optimized yet
 * @package WP-Favicons
 * @since 0.4.0
 * @author Edward de Leau <e@leau.net>, {@link http://edward.de.leau.net}
 * @copyright GPL 2
 *
 */
namespace leau\co\wp_favicons;
set_time_limit (0); // loading TLD takes some time
if (!class_exists("Database"))
{
	/**
	 * Database functions for wp-favicons
	 * @package WP-Favicons
	 * @since 0.4.0
	 * @author Edward de Leau <e@leau.net>, {@link http://edward.de.leau.net}
	 * @copyright GPL 2
	 */
	class Database
	{
		var $tld;
		// singleton instance
		private static $instance;

		// private constructor function
		// to prevent external instantiation
		private function __construct()
		{

		}
		private function __clone()
		{
		}

		// getInstance method
		public static function getInstance()
		{
			if(!self::$instance)
			{
				self::$instance = new self();
			}
			return self::$instance;
		}

		function Install()
		{
			global $wpdb;
			$instance = self::getInstance();
			$table_name_uri = $instance->TableNameUri();
			$table_name_icon = $instance->TableNameFavicon();

			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
;
			/* URI Tree */
			$sql = "CREATE TABLE $table_name_uri (
  					id bigint(20) NOT NULL AUTO_INCREMENT,
  					parent_id bigint(20) NOT NULL,
  					uri varchar(2048) COLLATE utf8_unicode_ci NOT NULL,
  					description varchar(255) COLLATE utf8_unicode_ci NOT NULL,
  					tld tinyint(1) NOT NULL,
  					publictld tinyint(1) NOT NULL,
  					haspublicnodes tinyint(1) NOT NULL,
  					itime timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  					PRIMARY KEY  (id)
				) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;";
			if( $wpdb->get_var("SHOW TABLES LIKE '$table_name_uri'") != $table_name_uri) {
				$wpdb->query( $sql );
			}
			dbDelta($sql);
			Config::AddOptions(Array('wpfavicons_uri_db_version' => '0.0.1'));

			/* Favicon Pointer*/
			$sql = "CREATE TABLE $table_name_icon (
  					id bigint(20) NOT NULL AUTO_INCREMENT,
  					parent_id bigint(20) NOT NULL,
  					favicon varchar(10) COLLATE utf8_unicode_ci NOT NULL,
  					filetypesure tinyint(1) NOT NULL,
  					filters varchar(255) COLLATE utf8_unicode_ci NOT NULL,
  					content varchar(255) COLLATE utf8_unicode_ci NOT NULL,
  					status varchar(25) COLLATE utf8_unicode_ci NOT NULL,
  					defaulticon tinyint(1) NOT NULL,
  					comment varchar(255) COLLATE utf8_unicode_ci NOT NULL,
  					itime timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  					PRIMARY KEY  (id)
				) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;";
			if( $wpdb->get_var("SHOW TABLES LIKE '$table_name_icon'") != $table_name_icon) {
				$wpdb->query( $sql );
			}
			dbDelta($sql);
			Config::AddOptions(Array('wpfavicons_icon_db_version' => '0.0.1'));

		}

		function TableNameUri()
		{
			global $wpdb;
			return $wpdb->prefix . "wpfavicons_uri";
		}

		function TableNameFavicon()
		{
			global $wpdb;
			return $wpdb->prefix . "wpfavicons_icon";
		}

		/**
		 * Loads the level 1, level 2 and level 3 TLD
		 * Since this takes some time during init this will be done by the developer
		 * and the output is loaded via a direct database load
		 * @package WP-Favicons
		 * @since 0.4.0
		 * @author Edward de Leau <e@leau.net>, {@link http://edward.de.leau.net}
		 * @copyright GPL 2
		 */
		function LoadTLD()
		{
			global $wpdb;
			$instance = self::getInstance();
			$instance->tld = new TLD();
			$table_name = $instance->TableNameUri();
			require_once("effectiveTLDs.inc.php");

			foreach($tldTree as $level1Key => $level1Value)
			{
				// get the domain e.g. 'ac'
				$level1domain = $level1Key;

				// check if it does not exist yet
				$row = $wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name WHERE uri = %s",
				$level1domain),OBJECT);
				if (!is_object($row)) {

					// if not insert
					$row = $wpdb->insert($table_name,array(
					 	'parent_id' => 0,
						'uri' => $level1domain,
						'description' => 'Level 1 official TLD',
						'tld' => true,
						'publictld' => false,
						'haspublicnodes' => true));
					// get the  key as parent id for children
					$level1_rowid = $wpdb->insert_id;
				}
				else
				{
					$level1_rowid = $row->id;
				}
				foreach($level1Value as $level2Key => $level2Value)
				{
					// LEVEL 2
					// !note that some $level2Key entries are * and ! we should handle them later
					$level2domain = $level2Key . '.' . $level1Key;
					// check if it does not exist yet
					$row = $wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name WHERE uri = %s",
					$level2domain),OBJECT);
					if (!is_object($row)) {
						// if not insert
						$row = $wpdb->insert($table_name,array(
					 		'parent_id' => $level1_rowid,
							'uri' => $level2domain,
							'description' => 'Level 2 official TLD',
							'tld' => true,
							'publictld' => false,
							'haspublicnodes' => true));
						// get the  key as parent id for children
						$level2_rowid = $wpdb->insert_id;
					}
					else
					{
						$level2_rowid = $row->id;
					}
					if ($level2Value)
					{
						foreach($level2Value as $level3Key => $level3Value)
						{
							$level3domain = $level3Key . '.' . $level2Key . '.' . $level1Key;
							$row = $wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name WHERE uri = %s",
							$level3domain),OBJECT);
							if (!is_object($row)) {
								// if not insert
								$row = $wpdb->insert($table_name,array(
						 		'parent_id' => $level2_rowid,
								'uri' => $level3domain,
								'description' => 'Level 3 official TLD',
								'tld' => true,
								'publictld' => false,
								'haspublicnodes' => true));
								// get the  key as parent id for children
								$level3_rowid = $wpdb->insert_id;
							}
						}
					}
				}
			}
		}

		function strstr_after($haystack, $needle, $case_insensitive = false) {
			$strpos = ($case_insensitive) ? 'stripos' : 'strpos';
			$pos = $strpos($haystack, $needle);
			if (is_int($pos)) {
				return substr($haystack, $pos + strlen($needle));
			}
			return $pos;
		}

		function CheckInsertUrl($uri)
		{
			$wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name WHERE uri = %s",
			$uri),OBJECT);
			if (!is_object($row)) {
				$row = $wpdb->insert($table_name,array(
					'parent_id' => $level2_rowid,
					'uri' => $level3domain,
					'description' => 'Level 3 official TLD ',
					'tld' => true,
					'publictld' => false,
					'haspublicnodes' => true,
				));
			}

		}

		function valid_ip($ip) {
			return preg_match("/^([1-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])" .
				"(\.([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])){3}$/", $ip);
		}

		/**
		 * How do we store URIs? (2,048 max length defined by IE)
		 *
		 * 1. we store the root domain seperately from the relative path
		 *    although in essence there is no difference in practice this
		 *    is handier
		 * 2. there are many "spcial relative paths" such as:
		 *    - http://www.youtube.com/user/
		 *    - http://apps.facebook.com/[NAME]
		 *
		 *    Which can be considered important nodes by themselves and could
		 *    just as well be subdomain. They are important because they hold
		 *    meta information on this level that is unique such as a favicon
		 *    per facebook application. In other words e.g. the Google S2 service
		 *    handles this only a domain level and thus has made a decision to
		 *    be very limited in the support of favicons.
		 *
		 * So what we need todo is build a hierarchical tree based on the uri
		 * structure where we in essence make no difference between subdomains
		 * or relative paths. Since we do not want to duplicate information there
		 * is a pattern behind the meta data attached within a certain family-line
		 * of urls. e.g. facebook.com has its own favicon and apps.facebook.com/whatever
		 * has its own favicon but none of the uri's on a deeper level.
		 * This however is ofcourse different per site since the developer of a site
		 * can decide how to form this structure.
		 * So we start with a certain set of websites that enable us to see the main
		 * patterns, for this we use the Alexa top 1.000.000 download csv site that
		 * shows us the top 1.000.000 most favorite sites. This is our test set since
		 * sites that do not fit in the pattern will drop out. E.g. right from the start
		 * we saw e.g. many IP addresses which we should take into account. We also see
		 * that Alexa sometimes takes a root domain as root for their ranking and other
		 * times a relative path. So we need to form a library of patterns we meet.
		 *
		 * I am still thinking about the architecture behind this. Because in time we
		 * want to tie these uri's to concepts such as tags and categories in the
		 * WordPress database to create a sort of WordPress Social graph. E.g. "book"
		 * should be related to all uri's about book as well as your profile's on
		 * book social networking sites. At then end we would want a certain local
		 * URI to display all outgoing URI's that relate to that local URI. I think
		 * that is main goal for this plugin. This is handy because the local tags
		 * would immediately get interconnected with a mich wider span of URI's and
		 * it gets more meaning and more connections.
		 *
		 * One main problem are non functional requirements. This ofcourse will lead
		 * to a HUGE database and much processing. Basically you are capturing every
		 * possible URI on the Internet... and not only that but also the relation
		 * between each URI with each other URI............ (^n).
		 *
		 * The main pattern is to find out when a certain group of sub-tree belongs
		 * to one entity and for now it seems that i can only do this by maintaining
		 * lists of domains that allow fanning out in relative paths to find more
		 * 'interesting nodes' or 'master nodes'.
		 *
		 * So... I take the pragmatic approach here.
		 *
		 * 0.4.0 I will just start with domains only in this version
		 * 0.5.0 Will then extend it to important nodes in relative urls
		 * 0.6.0 Will then try to extend it to all relative urls and files
		 *
		 * also notice that we only take domain and paths until 0.6.0 while obviously
		 * protocol (e.g. https and http could have different favicons),
		 * username/password etc... should also be in the database as parts of the tree.
		 * (http://php.net/manual/en/function.parse-url.php) I just hope this covers
		 * the majority of cases for now. We are just a simple plugin not Google.
		 *
		 * @param $uri
		 */
		function DbUrlCheck($uri)
		{
			$instance = self::getInstance();
			global $wpdb;
			$table_name = $instance->TableNameUri();
			$url_host_path = $instance->getFullPath($uri);
			$parent_id=0;

			// first check the full domain incl path if that already exists skip the rest
			$row = $wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name WHERE uri = %s",
			$url_host_path),OBJECT);
			if (is_object($row)) {
				return true;
			}

			// -------------------------------------------
			// BEGIN: IP ADDRESS
			// note that dynamic IP addresses give much conflicts
			// let alone 127.0.0.1...
			// This is not for IP that related to dns names but for IP
			// that have no TLD derived dns name
			// -------------------------------------------
			$url_host = parse_url($uri, PHP_URL_HOST);
			if ($instance->valid_ip($url_host))
			{
				$row = $wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name WHERE `uri` = '%s'",
				trim($url_host)),OBJECT);
				if (!is_object($row)) {
					$row = $wpdb->insert($table_name,array(
						'parent_id' => 0,
						'uri' => $url_host,
						'description' => '',
						'tld' => false,
						'publictld' => false,
						'haspublicnodes' => false,
					));
					$parent_id = $wpdb->insert_id;
				}
				else
				{
					$parent_id = $row->id;
				}
			// -------------------------------------------
			// END: IP ADDRESS
			// -------------------------------------------
			}
			else
			{

				// -------------------------------------------
				// BEGIN: domain parts:
				// -------------------------------------------
				$domainparts = array();

				// first check the full domain if that already exists skip the rest
				$row = $wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name WHERE uri = %s",
				$url_host),OBJECT);
				if (!is_object($row)) {
					// if it does not exist check parents
					$url_host = '.' . $url_host;
					while ( ($url_host = $instance->strstr_after($url_host, '.', true)) )
					{
						$domainparts[] = $url_host;
					}
					// we check for the existence of the domain starting from the bottom level
					// theoretically all TLD should be there already but you never know.
					//
					$parent_id = 0;
					foreach(array_reverse($domainparts) as $domainpart)
					{
						// check if DOMAINPART exists
						$row = $wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name WHERE `uri` = '%s'",
						trim($domainpart)),OBJECT);
						if (!is_object($row)) {
							$row = $wpdb->insert($table_name,array(
						'parent_id' => $parent_id,
						'uri' => $domainpart,
						'description' => '',
						'tld' => false,
						'publictld' => false,
						'haspublicnodes' => false,
						));
							$parent_id = $wpdb->insert_id;
						}
						else
						{
							$parent_id = $row->id;
						}
					}
				}
				else
				{
					$parent_id = $row->id;
				}
				// -------------------------------------------
				// END OF : domain parts:
				// -------------------------------------------
			}

			// -------------------------------------------
			// BEGIN OF : ports
			// -------------------------------------------
			$url_host = parse_url($uri, PHP_URL_HOST);
			$url_port = parse_url($uri, PHP_URL_PORT);
			$domainpart = $url_host . ':' . $url_port;
			if ($url_port)
			{
				// check if it exists
				$row = $wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name WHERE `uri` = '%s'",
					trim($domainpart)),OBJECT);
				if (!is_object($row)) {
				$row = $wpdb->insert($table_name,array(
					'parent_id' => $parent_id,
					'uri' => $domainpart,
					'description' => '',
					'tld' => false,
					'publictld' => false,
					'haspublicnodes' => false,
					));
					$parent_id = $wpdb->insert_id;
				}
				else
				{
					$parent_id = $row->id;
				}
			}
			else
			{
				// maybe insert :80 on all entries
			}

			// -------------------------------------------
			// END OF : ports
			// -------------------------------------------

			// -------------------------------------------
			// BEGIN OF : paths
			// notice that we still have the parent id above
			// we put this in already for future versions
			// -------------------------------------------
			// e.g.: /wp2/wp-admin/post.php

			$url_host = parse_url($uri, PHP_URL_HOST);
			$url_port = parse_url($uri, PHP_URL_PORT);
			if ($url_port)
			{
				$firstbit = $url_host . ':' . $url_port;
			}
			else
			{
				$firstbit = $url_host;
			}

			$url_path = parse_url($uri, PHP_URL_PATH);
			$url_path = substr($url_path,1);
			if ($url_path!='')
			{
				$domainpathparts = array();
				$path_until_now = '';
				$left='';
				$right=$url_path;
				while ( strstr($url_path, '/', true) )
				{
					$left = strstr($url_path, '/', true);
					$path_until_now .= '/' . $left;
					$domainpathparts[] = $path_until_now;
					$right = substr(strstr($url_path, '/', false),1);
					$url_path = $right;
				}
				$domainpathparts[] = $path_until_now . '/' . $right;
				foreach($domainpathparts as $domainpathpart)
				{
					$domainpart = $firstbit . $domainpathpart;
					$row = $wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name WHERE `uri` = '%s'",
						trim($domainpart)),OBJECT);
					if (!is_object($row)) {
						$row = $wpdb->insert($table_name,array(
						'parent_id' => $parent_id,
						'uri' => $domainpart,
						'description' => '',
						'tld' => false,
						'publictld' => false,
						'haspublicnodes' => false
						));
						$parent_id = $wpdb->insert_id;
					}
					else
					{
						$parent_id = $row->id;
					}
				}
			}
			// -------------------------------------------
			// END OF : paths
			// -------------------------------------------
			// update the record to show that this one is in the users blog
			$row = $wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name WHERE `id` = '%d'",
				$parent_id),OBJECT);


			if (is_object($row)) {
				$row = $wpdb->update($table_name,
						array( 'haspublicnodes' => 1 ),
						array( 'id' => $parent_id ),
						array ( '%d'),
						array ( '%d')
				);
			}

		}

		/**
		 * Gets the path information
		 * used by multiple functions
		 * @param string $uri the URI to set parameters for
		 */
		function getFullPath($uri)
		{
			$instance = self::getInstance();
			$url_host = parse_url($uri, PHP_URL_HOST);
			$url_port = parse_url($uri, PHP_URL_PORT);
			if ($url_port) {$url_port=':'.$url_port;}
			$url_path = parse_url($uri, PHP_URL_PATH);
			if ($url_path=='/')
			{
				$url_path = '';
			}
			$url_host_path = $url_host . $url_port . $url_path;
			return $url_host_path;
		}

		/*
		 * Logs a found favicon in the database
		 * @param $faviconArray
		 */
		function LogFaviconFound($faviconArray)
		{
			/* general init for db functions */
			$instance = self::getInstance();
			global $wpdb;
			$table_name_uri = $instance->TableNameUri();
			$table_name_icon = $instance->TableNameFavicon();
			$url_host_path = $instance->getFullPath($faviconArray['uri']);

			/* check if the uri exists in the uri table */
			$row = $wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name_uri WHERE `uri` = '%s'",
				trim($url_host_path)),OBJECT);
			if (is_object($row)) {
				/* Always insert new entries so we have a history */
				$row = $wpdb->insert($table_name_icon,
					array(
						'parent_id' => $row->id,
						'favicon' => $faviconArray['type'],
						'filetypesure' => $faviconArray['filetypesure'],
						'content' => $faviconArray['source'],
						'status' => 'FOUND',
						'defaulticon' => $faviconArray['default'],
						'comment' =>  $faviconArray['comment']
				));
				$id = $wpdb->insert_id;
			}
			else
			{
				// uri could not be found @todo 0.5.0 log Error
				// this is an orphan
				$id=0;
			}
			return $id;
		}

		/*
		 * Update a found favicon in the database with filters applied
		 * @param $faviconArray
		 */
		function UpdateFaviconFoundWithFilter($filterName,$id,$comment='',$filetype)
		{
			$instance = self::getInstance();
			global $wpdb;
			$table_name_uri = $instance->TableNameUri();
			$table_name_icon = $instance->TableNameFavicon();

			$row = $wpdb->get_row( $wpdb->prepare("SELECT `parent_id`, `filters`, `defaulticon`, `comment`, `content` FROM $table_name_icon WHERE `id` = '%s'",
				$id,OBJECT));

			// update the object with a new filter name
			if (is_object($row)) {
				$row = $wpdb->insert($table_name_icon,
					array(
						'parent_id' => $row->parent_id,
						'favicon' => $filetype,
						'filetypesure' => true,
						'content' => $row->content,
						'status' => 'FILTER',
						'filters' => $row->filters . $filterName,
						'defaulticon' => $row->defaulticon,
						'comment' => $row->comment . $comment
				));
			}
			else
			{
				// uri could not be found @todo 0.5.0 log Error
				// could not log Update by Filter in database
			}
			return;
		}

		/**
		 * Get information about the stored favicon
		 * During display the only info we have is the url
		 * @param string $url
		 */
		function GetIconInfo($url)
		{
			$instance = self::getInstance();
			global $wpdb;
			$url_host_path = $instance->getFullPath($url);
			$table_name_uri = $instance->TableNameUri();
			$table_name_icon = $instance->TableNameFavicon();

			$row = $wpdb->get_row( $wpdb->prepare("SELECT id FROM $table_name_uri WHERE uri = %s",
				$url_host_path),OBJECT);
			if (!is_object($row)) {
				return false;
			}
			$id = $row->id;
			$row = $wpdb->get_row( $wpdb->prepare("SELECT `favicon`,`content`,`defaulticon` FROM $table_name_icon "
			. "WHERE parent_id = %d AND (status='FOUND' OR status='FILTER') ORDER BY itime desc;",$id),OBJECT);

			if (!is_object($row)) {
				return false;
			}
			return $row;
		}

		function EmptyIconCache()
		{
			global $wpdb;
			$instance = self::getInstance();
			$table_name_icon = $instance->TableNameFavicon();

			$wpdb->query("DELETE FROM $table_name_icon;");
		}

	}
}


if (!class_exists("TLD"))
{
	/*
	 * external source, license to be added
	 */
	class TLD
	{

		function __construct()
		{

		}

		function getRegisteredDomain($signingDomain)
		{
			global $tldTree;
			$signingDomainParts = explode('.', $signingDomain);
			$result = $this->findRegisteredDomain($signingDomainParts, $tldTree);

			if ($result===NULL || $result=="") {
				// this is an invalid domain name
				return NULL;
			}

			// assure there is at least 1 TLD in the stripped signing domain
			if (!strpos($result, '.')) {
				$cnt = count($signingDomainParts);
				if ($cnt==1 || $signingDomainParts[$cnt-2]=="") return NULL;
				return $signingDomainParts[$cnt-2].'.'.$signingDomainParts[$cnt-1];
			}
			return $result;
		}

		// recursive helper method
		function findRegisteredDomain($remainingSigningDomainParts, &$treeNode) {

			$sub = array_pop($remainingSigningDomainParts);

			$result = NULL;
			if (isset($treeNode['!'])) {
				return '#';
			} else if (is_array($treeNode) && array_key_exists($sub, $treeNode)) {
				$result = $this->findRegisteredDomain($remainingSigningDomainParts, $treeNode[$sub]);
			} else if (is_array($treeNode) && array_key_exists('*', $treeNode)) {
				$result = $this->findRegisteredDomain($remainingSigningDomainParts, $treeNode['*']);
			} else {
				return $sub;
			}

			// this is a hack 'cause PHP interpretes '' as NULL
			if ($result == '#') {
				return $sub;
			} else if (strlen($result)>0) {
				return $result.'.'.$sub;
			}
			return NULL;
		}
	}
}

