<?php
/**
 * Filter to Extract a favicon from a webpage
 *
 * This file contains the wp_favicons_search filter implementation
 * to get a webpage from the and extract the icon from there.
 *
 * Note: feel free to add your own filters!
 *
 * @package WP-Favicons
 * @since 0.4.0
 * @author Edward de Leau <e@leau.net>, {@link http://edward.de.leau.net}
 * @copyright GPL 2
 */

namespace leau\co\wp_favicons;

if (!class_exists("sources_page"))
{
	/**
	 * Filter to Extract a favicon from a webpage
 	 * @package WP-Favicons
 	 * @since 0.4.0
	 */
	class sources_page extends Plugin
	{
		public $_data_url = '';

	 	/**
    	 * (non-PHPdoc)
    	 * @see plugins/EdlLinkFaviconPlugin::AddFilter()
    	 */
    	function AddFilter()
    	{
    		/* parent class arranges that this is not called when the bool is off */
    		add_filter(Config::GetPluginSlug() . 'search', array($this,'DoFilter'), 6, 2);
    	}



		/**
		 *
		 * Enter description here ...
		 * @param $favicon_array
		 * @param $url
		 */
		function DoFilter( /*array*/ $favicon_array, /*string*/ $url)
		{
			$data_url = $this->DataUrl($url);
			/* the following three lines should always be the executed */
			if (!$data = EdlUtils::favicon_filter_init($favicon_array, $data_url)) { return $favicon_array; }
			$favicon_array = $this->ExecuteFilter($favicon_array, $url, $data);
			return $favicon_array;
		}

		/**
 		*
 		* Enter description here ...
 		* @param $url
 		*/
		function j_parseUrl($url) {
  			$r  = "(?:([a-z0-9+-._]+)://)?";
  			$r .= "(?:";
  			$r .=   "(?:((?:[a-z0-9-._~!$&'()*+,;=:]|%[0-9a-f]{2})*)@)?";
  			$r .=   "(?:\[((?:[a-z0-9:])*)\])?";
  			$r .=   "((?:[a-z0-9-._~!$&'()*+,;=]|%[0-9a-f]{2})*)";
  			$r .=   "(?::(\d*))?";
  			$r .=   "(/(?:[a-z0-9-._~!$&'()*+,;=:@/]|%[0-9a-f]{2})*)?";
  			$r .=   "|";
  			$r .=   "(/?";
  			$r .=     "(?:[a-z0-9-._~!$&'()*+,;=:@]|%[0-9a-f]{2})+";
  			$r .=     "(?:[a-z0-9-._~!$&'()*+,;=:@\/]|%[0-9a-f]{2})*";
  			$r .=    ")?";
  			$r .= ")";
  			$r .= "(?:\?((?:[a-z0-9-._~!$&'()*+,;=:\/?@]|%[0-9a-f]{2})*))?";
  			$r .= "(?:#((?:[a-z0-9-._~!$&'()*+,;=:\/?@]|%[0-9a-f]{2})*))?";
  			preg_match("`$r`i", $url, $match);
  			$parts = array(
            	"scheme"=>'',
            	"userinfo"=>'',
            	"authority"=>'',
            	"host"=> '',
            	"port"=>'',
            	"path"=>'',
            	"query"=>'',
            	"fragment"=>'');
  			switch (count ($match))
  			{
    			case 10: $parts['fragment'] = $match[9];
    			case 9: $parts['query'] = $match[8];
    			case 8: $parts['path'] =  $match[7];
    			case 7: $parts['path'] =  $match[6] . $parts['path'];
    			case 6: $parts['port'] =  $match[5];
    			case 5: $parts['host'] =  $match[3]?"[".$match[3]."]":$match[4];
    			case 4: $parts['userinfo'] =  $match[2];
    			case 3: $parts['scheme'] =  $match[1];
  			}
  				$parts['authority'] = ($parts['userinfo']?$parts['userinfo']."@":"").
                	$parts['host'].
                    ($parts['port']?":".$parts['port']:"");
  			return $parts;
		}

		/**
 		 *
 		 * Gets the full image path for a found image in a link
 		 *
 	 	 * @param $url
 		 * @param $image
 		 */
		function getFullFaviconPath($url, $image)
		{
			$image_parts = $this->j_parseUrl($image);
			if ($image_parts['scheme']) {
				EdlUtils::favicon_page_msg($url,' [FROMPAGE] trying: ' . $image);
				return $image;
			}
			$arr_parts = $this->j_parseUrl($url);
			$root1 = '';
			if ($arr_parts['scheme'])
			{
				$root1.=$arr_parts['scheme'].'://';
			}
			if ($arr_parts['userinfo'])
			{
				$root1.=$arr_parts['userinfo'].'@';
			}
			if ($arr_parts['host'])
			{
				$root1.=$arr_parts['host'].':';
			}
			if ($arr_parts['port'])
			{
				$root1.=$arr_parts['root'];
			}
			else
			{ // no port specified; get default port
				if ($arr_parts['scheme'])
				{
					switch( $arr_parts['scheme'] )
					{
						case 'http':
							$port = 80; // default for http
						break;
						case 'https':
							$port = 443; // default for https
						break;
						case 'ftp':
							$port = 21; // default for ftp
						break;
						case 'ftps':
							$port = 990; // default for ftps
						break;
						default:
							$port = 0; // error; unsupported scheme
						break;
					}
				}
				else
				{
					$port = 0; // error; unknown scheme
				}
				$root1.=$port;
			}
			if ($image[0]=='/' || $image[0]=='\\' )
			{
				return $root1 . $image;
			}
			//
			// the last possibility is that the favicon is defined as e.g. 'favicon.ico"
			// and the url is e.g. http://x.x/whatever/whatever.rr
			// meaning: the user has a different favicon for each single page............
			//
			// it is very hard to determine if whatever.rr is a file or a directory
			// SO... I could grab it, look into it and see if it a file... and is different than the
			// file on http://x.x/whatever/whatever.rr/ but... that is probably not funny.
			//
			// SO... i will return false for that case. In case someone wants to write an extended filter
			// for that case: be my guest
			return false;
		}

		/**
 		 *
 		 * Enter description here ...
 		 * @param unknown_type $xpath
 		 * @param unknown_type $XpathExpr
 		 * @param unknown_type $url
 		 */
		function WpFavIconXpathHrefCheck($xpath, /*string*/ $XpathExpr, /*string*/ $url)
		{
			$elements = $xpath->query($XpathExpr);

			if (!is_null($elements))
			{
				foreach ($elements as $element)
				{
					if ($element->getAttribute('href'))
					{
						$href = $element->getAttribute('href');
						return $href;
					}
				}
			}
			return false;
		}

		/**
 		 *
 		 * Enter description here ...
 		 * @param $favicon_array
 		 * @param $imagepath
 		 */
		function WPFaviconDownloadImage( /*array*/ $favicon_array, /*string*/ $imagepath)
		{
			if ($imagepath)
			{
				if (!$data = EdlUtils::favicon_filter_init($favicon_array, $imagepath))
				{
					return false;
				}
				else
				{
					/* check that data is an icon not a redirected page */
					if (strpos(strtolower($data),strtolower('<html>'))) {
						if (strpos(strtolower($data),strtolower('<body>'))) {
							return $favicon_array;
						}
					}
					/*
					 * @todo 0.5.0 this stuff will not work with pages such as daylife.com
					 * which generate the code from javascript
					 *
					 */

					$favicon_array['data'] = $data;
					return $favicon_array;
				}
			}
			return $favicon_array;
		}

		/**
 		 *
 		 * Enter description here ...
 		 * @param $xpath
 		 * @param $XpathExpr
 		 * @param $url
 		 * @param $favicon_array
 		 */
		function WpFavIconGetIcon( $xpath, /*string*/ $XpathExpr, /*string*/ $url, $favicon_array )
		{
			if ($imagepath=$this->WpFavIconXpathHrefCheck($xpath, $XpathExpr, $url))
			{
				EdlUtils::favicon_page_msg($url,' [FROMPAGE] pass 1');

				if (substr(trim($imagepath),0,4)=='data')
				{
					/*
					 * if the icon is encoded e.g. like: data:image/x-icon;base64,iVBORw0==
					 * then get it
					 */
					list($type,$rest) = explode(":", $imagepath);
					list($mimetype,$rest) = explode(";",$rest);
					list($encoding,$imagedata) = explode(",", $rest);
					$type = trim($type);
					$mimetype = trim($mimetype);
					$encoding = trim($encoding);
					$imagedata = trim($imagedata);

					/* I guess a lot of decodings can theoretically be used ... lets see
					 * what we encounter in practice */
					if ($encoding=='base64')
					{
						$favicon_data = base64_decode($imagedata);
					}
					else
					{
						// @todo 0.5.0 unsupported encoding embedded
						$favicon_array='';
						return $favicon_array;
					}

					EdlUtils::favicon_page_msg($url,' [FROMPAGE] found embedded icon');
					/* verify the image */
					$favicon_array['type'] = '';
					$favicon_array['data'] = $favicon_data;
					$favicon_array = EdlUtils::VerifyImage($favicon_array);

					$id=Database::LogFaviconFound(array(
				 		'uri' => $url,
						'type' => $favicon_array['type'],
						'filetypesure' => true,
						'data' => $favicon_data,
						'default' => false,
						'comment' => $url . ' (embedded)',
						'source'=> $XpathExpr));
					$favicon_array['id'] = $id;

					return $favicon_array;
				}
				else
				{
					/*
					 * it is found to be embedded and not an embedded data string
					 */
					$imagepath = $this->getFullFaviconPath($url, $imagepath);
					if ($favicon_array = $this->WPFaviconDownloadImage($favicon_array,$imagepath))
					{
						EdlUtils::favicon_page_msg($url,' [FROMPAGE] downloaded linked image');
						// fix for weird extensions:
						$favicon_array['type'] = EdlUtils::GetExtension($imagepath);

						/* verify the image */
						$favicon_array = EdlUtils::VerifyImage($favicon_array);

						$id=Database::LogFaviconFound(array(
				 			'uri' => $url,
							'type' => $favicon_array['type'],
							'filetypesure' => true,
							'data' => $favicon_array['data'],
							'default' => false,
							'comment' => $url,
							'source'=> $XpathExpr));
						$favicon_array['id'] = $id;
						return $favicon_array;
					}
					else
					{
						EdlUtils::favicon_page_msg($url,' [FROMPAGE] could not download linked image');
					}
				}
			}
			return false;
		}


	    /**
    	 * (non-PHPdoc)
    	 * @see plugins/search/inc/EdlLinkFaviconPluginSources::DataUrl()
    	 */
    	function DataUrl($url)
    	{
    		$this->_data_url = $url;
			return $this->_data_url;
    	}

		/**
 		 *
 		 * Enter description here ...
 		 * @param $favicon_array
 		 * @param $url
 		 */
		function ExecuteFilter( /*array*/ $favicon_array, /*string*/ $url, $data)
		{
			/*
	 		 * PHP DOM parser is more intuitive than e.g.
	 		 * '/<link\s[^>]*rel\s*=\s*\'shortcut\sicon\'\s[^>]*href\s*=\s*"(.*)[^>]*">/iU',
	 		 * ! (although it might be slightly slower)
	 		 */
			$dom = new \DOMDocument();
			libxml_use_internal_errors(true);
			$dom->strictErrorChecking = false;
			if (!$dom->loadHTML($data))
			{
				/*
				 * if page gets loaded as XML check if it has errors
				 */
				foreach (libxml_get_errors() as $error)
				{
					// 	handle errors here
				}
				libxml_clear_errors();
			}
			else
			{
				/*
				 * Try to find either icon or shortcut icon in the page
				 */
				$xpath = new \DOMXPath($dom);
				if ($favicon_array=$this->WpFavIconGetIcon($xpath, "/html/head/link[@rel='shortcut icon']", $url, $favicon_array ))
				{
					EdlUtils::favicon_page_msg($url,' [FROMPAGE] FOUND!');
					return $favicon_array;
				}
				if ($favicon_array=$this->WpFavIconGetIcon($xpath, "/html/head/link[@rel='icon']", $url, $favicon_array ))
				{
					EdlUtils::favicon_page_msg($url,' [FROMPAGE] FOUND!');
					return $favicon_array;
				}
				/*
				 * 0.4.4: app.facebook pages show icon in body, fixed, see:
				 * http://stackoverflow.com/questions/5162045/why-is-this-xpath-query-not-working-on-the-dom-of-facebook-application-pages
				 */
				if ($favicon_array=$this->WpFavIconGetIcon($xpath, "//link[@rel='shortcut icon']", $url, $favicon_array ))
				{
					EdlUtils::favicon_page_msg($url,' [FROMPAGE] FOUND!');
					return $favicon_array;
				}

			}
			EdlUtils::favicon_page_msg($url,' [FROMPAGE] NOT FOUND');
			return $favicon_array;
		}
	}
}
